/*
  Demo of RSA signature checking, from a certificate.
  
  Dr. Orion Sky Lawlor, lawlor@alaska.edu, 2013-03-06 (Public Domain)
*/
#include <iostream>
#include "bigint.h"
#include "bigint.cpp" /* for linking easier */

// Stringify this argument, wrapping it in quotes.
#define QUOTE(x) #x

int main(int argc,char *argv[]) {
	// Pick public key: Google Internet Authority
	BigInteger n; // public key modulus
	n.readHex(QUOTE(
C9 ED B7 A4 8B 9C 57 E7 84 3E 40 7D 84 F4 8F D1
71 63 53 99 E7 79 74 14 AF 44 99 33 20 92 8D 7B
E5 28 0C BA AD 6C 49 7E 83 5F 34 59 4E 0A 7A 30
CD D0 D7 C4 57 45 ED D5 AA D6 73 26 CE AD 32 13
B8 D7 0F 1D 3B DF DD DC 08 36 A8 6F 51 44 9B CA
D6 20 52 73 B7 26 87 35 6A DB A9 E5 D4 59 A5 2B
FC 67 19 39 FA 93 18 18 6C DE DD 25 8A 0E 33 14
47 C2 EF 01 50 79 E4 FD 69 D1 A7 C0 AC E2 57 6F
));
	BigInteger e=65537;  // public key exponent
	
	// Pick message: signature of Google web server
	BigInteger c; 
	c.readHex(QUOTE(
9B E8 97 20 CC 16 89 E5 6C CC E5 30 49 E5 75 C1
66 4D 1A C2 76 82 E5 DF CA 40 DA 00 12 40 BE 45
63 80 B0 30 8C DD C3 CA 1D BD AF 72 3C F3 62 2F
B1 B0 76 5D 94 FD 77 71 79 63 E3 73 FE F1 D7 D8
32 7D 96 AF 56 78 38 28 61 97 7E A2 99 6B 1C 3B
B1 C1 6A 4A ED 29 FF F9 78 12 6A FA 43 47 9C 71
45 8A 1A 57 22 CD 4B 16 15 D2 D5 CC B9 E1 4F 86
6F 14 B4 DB 2E 39 4E 1C D2 9D BB 17 B3 C3 89 88
));
	std::cout<<"n="<<n.hex()<<"\n";
	std::cout<<"c="<<c.hex()<<"\n";
	
	// Check signature (decode message)
	BigInteger s=c.modPow(e,n);
	
	// Print signed value--
	//    Starts with 0x0001
	//    lots of 0xff
	//    one 0x00
	//    DER-encoded 
	std::cout<<"s="<<s.hex()<<"\n";
	
	return 0;
}

