/*
  A toy implementation of Diffie-Hellman-Merkle key exchange.

  CAUTION -- QUITE INSECURE: 
    - Uses srandom/random to generate primes (fail)
    - Unusably slow for secure-size primes, like 1000 bits

  For classroom use only.
  
  Dr. Orion Sky Lawlor, lawlor@alaska.edu, 2013-02-28 (Public Domain)
*/
#include <iostream>
#include "bigint.h"
#include "bigint.cpp" /* for linking easier */

int main(int argc,char *argv[]) {
	if (argc>1) srandom(atoi(argv[1]));
	
	int bits=30; // number of bits in each prime
	if (argc>2) bits=atoi(argv[2]);

for (int reps=0;reps<100;reps++) {
	srandom(reps);

	// Generate shared public values:
#if 1
// FAIL: small random generators often result in a secret key of 0x1.
	BigInteger p=BigInteger::probablePrime(bits); // publish
	BigInteger g=BigInteger::random(bits/2)+2;  // FIXME: find generator
#else
// Strong prime trick (much slower, still not perfect)
	BigInteger p;
	do {
		p=BigInteger::probablePrime(bits);
		p=p*2+1; // make p a "strong prime"
	} while (!p.isProbablePrime());
	BigInteger g=2; // now 2 is a generator
#endif
	
	// side A:
	BigInteger a=BigInteger::random(bits/2)+1;
	BigInteger A=g.modPow(a,p);
	
	// side B:
	BigInteger b=BigInteger::random(bits/2)+1;
	BigInteger B=g.modPow(b,p);
	
	// .. exchange A and B across network here ...
	
	// Compute private keys
	BigInteger SA=A.modPow(b,p); // on B side
	BigInteger SB=B.modPow(a,p); // on A side

	if (SA!=SB || SA==1 || (reps%10==0)) { 
		std::cout<<"   after "<<reps<<" iterations:\n";

		// Print results
		std::cout<<"Prime p: "<<p<<"\n";
		std::cout<<"Base  g: "<<g<<"\n";
		std::cout<<"Secret a: "<<a<<"\n";
		std::cout<<"Secret b: "<<b<<"\n";
		std::cout<<"g^a: "<<A<<"\n";
		std::cout<<"g^b: "<<B<<"\n";
		std::cout<<"Shared secret A^b: 0x"<<SA.hex()<<"\n";
		std::cout<<"Shared secret B^a: 0x"<<SB.hex()<<"\n";

		if (SA != SB) {
			std::cout<<"BAD";
			break;
		}
	}
}
	
	
	return 0;
}

