/*
  A toy implementation of Diffie-Hellman-Merkle key exchange.

  CAUTION -- QUITE INSECURE: 
    - Uses srandom/random to generate primes (fail)
    - Unusably slow for secure-size primes, like 1000 bits

  For classroom use only.
  
  Dr. Orion Sky Lawlor, lawlor@alaska.edu, 2013-02-28 (Public Domain)
*/
#include <iostream>
#include "ecc_lib.h"
#include "ecc_lib.cpp" /* for linking easier */


int main(int argc,char *argv[]) {
	if (argc>1) srandom(atoi(argv[1]));
	
	int bits=128; // number of bits in each scalar integer
	if (argc>2) bits=atoi(argv[2]);

	// Pick a shared elliptic curve
	ECcurve_NISTP256 curve; // NIST P-256
	//ECcurve_BitCoin curve; // SECG p256k1

for (int reps=0;reps<=10;reps++)
{
	// side A:
	BigInteger a=BigInteger::random(bits);
	ECpoint A=curve.start.multiply(a,curve);
	
	// side B:
	BigInteger b=BigInteger::random(bits);
	ECpoint B=curve.start.multiply(b,curve);
	
	// .. exchange A and B across network here ...
	
	// Compute private keys
	ECpoint SA=A.multiply(b,curve); // on B side
	ECpoint SB=B.multiply(a,curve); // on A side

	if (SA!=SB || (reps%10==0)) { 
		std::cout<<"   after "<<reps<<" iterations:\n";

		// Print results
		std::cout<<"Secret a: "<<a<<"\n";
		std::cout<<"Secret b: "<<b<<"\n";
		std::cout<<"g^a: "<<A<<"\n";
		std::cout<<"g^b: "<<B<<"\n";
		std::cout<<"Shared secret A^b: "<<SA<<"\n";
		std::cout<<"Shared secret B^a: "<<SB<<"\n";

		if (SA!=SB) {
			std::cout<<"BAD!";
			break;
		}
	}
}
	
	
	return 0;
}

