/**
  A trivial network client doing some elliptic curve crypto authentication.
  
  Dr. Orion Sky Lawlor, lawlor@alaska.edu, 2013-04-10 (Public Domain)
*/
#include "bigint.h"
#include "ecc_lib.h"
#include "osl/socket.h"

#include "ecc_lib.cpp"
#include "osl/socket.cpp"

int main() {
	srandom(6); // <sarcasm> nobody could ever guess... six! </sarcasm>	
	
	unsigned int port=12345;
	std::cout<<"Client: Connecting to server on port "<<port<<"\n";
	SOCKET s=skt_connect(skt_lookup_ip("sandy.cs.uaf.edu"),port,10);
	std::cout<<"Client: Connected to server.  Starting exchange.\n";
	
	// Elliptic Curve Diffie-Hellman Key Exchange:	
	ECcurve_BitCoin curve; // pick a common curve
	std::string protocol="ECDH-BitCoin";
	skt_send_line(s,protocol); // send ours to server
	if (skt_recv_line(s)!=protocol) { // check server's protocol
		std::cout<<"Client: Protocol mismatch!\n";
		skt_close(s);
		exit(1);
	}
	
	// Make our half of the secret:
	int bits=256; // length of secret key
	BigInteger B=BigInteger::random(bits);
	ECpoint GB=curve.start.multiply(B,curve);
	
	// Send GB to the server.
	skt_send_line(s,GB.x.hex());
	skt_send_line(s,GB.y.hex());
	
	// Receive GA from the server.
	ECpoint GA;
	GA.x.readHex(skt_recv_line(s));
	GA.y.readHex(skt_recv_line(s));
	
	// Compute GAB by advancing server's point by our secret.
	ECpoint GAB=GA.multiply(B,curve);
	
	// We now have a shared secret--stupidly print it out.
	//  (In reality, this could be the encryption key.)
	std::cout<<"Client: shared secret is "<<GAB.x.hex()<<"\n";
	
	skt_close(s);	
	return 0;
}
