/**
  A trivial network server doing some elliptic curve crypto authentication.
  
  Dr. Orion Sky Lawlor, lawlor@alaska.edu, 2013-04-10 (Public Domain)
*/
#include "bigint.h"
#include "ecc_lib.h"
#include "osl/socket.h"

#include "ecc_lib.cpp"
#include "osl/socket.cpp"

int main() {
	srandom(3); // <sarcasm> nobody could ever guess... three! </sarcasm>	
	
	unsigned int port=12345;
	SERVER_SOCKET serv=skt_server(&port);
	std::cout<<"Server: Listening on port "<<port<<"\n";
	
	SOCKET s; // socket for talking to the current client
	while (s=skt_accept(serv,NULL,NULL)) 
	{
		std::cout<<"Server: client connected.  Starting exchange.\n";

		// Elliptic Curve Diffie-Hellman Key Exchange:	
		ECcurve_BitCoin curve; // pick a common curve
		std::string protocol="ECDH-BitCoin";
		skt_send_line(s,protocol); // send ours to client
		if (skt_recv_line(s)!=protocol) { // check client's protocol
			std::cout<<"Server: Protocol mismatch!\n";
			skt_close(s);
			continue;
		}
		
		// Make our half of the secret:
		int bits=256; // length of secret key
		BigInteger A=BigInteger::random(bits);
		ECpoint GA=curve.start.multiply(A,curve);
		
		// Send GA to the client.
		skt_send_line(s,GA.x.hex());
		skt_send_line(s,GA.y.hex());
		
		// Receive GB from the client.
		ECpoint GB;
		GB.x.readHex(skt_recv_line(s));
		GB.y.readHex(skt_recv_line(s));
		
		// Compute GAB by advancing client's point by our secret.
		ECpoint GAB=GB.multiply(A,curve);
		
		// We now have a shared secret--stupidly print it out.
		//  (In reality, this could be the encryption key.)
		std::cout<<"Server: shared secret is "<<GAB.x.hex()<<"\n";
		
		skt_close(s);
	}
	
	return 0;
}
