/*
  A toy implementation of RSA encryption.

  CAUTION -- TOTALLY INSECURE: 
    - Uses srandom/random to generate primes (fail)
    - Prints the primes found (FAIL!)
    - Unusably slow for real prime sizes like 500 bits

  For classroom use only.
  
  Dr. Orion Sky Lawlor, lawlor@alaska.edu, 2013-02-25 (Public Domain)
*/
#include <iostream>
#include "bigint.h"
#include "bigint.cpp" /* for linking easier */

int main(int argc,char *argv[]) {
	if (argc>1) srandom(atoi(argv[1]));
	
	int bits=30; // number of bits in each prime
	if (argc>2) bits=atoi(argv[2]);
	
	// Generate p and q
	BigInteger p=BigInteger::probablePrime(bits); // SHRED!
	BigInteger q=BigInteger::probablePrime(bits); // SHRED!
	BigInteger n=p*q; // publish
	BigInteger e=65537;  // publish
	BigInteger d=e.modInverse((p-1)*(q-1)); // our secret key
	
	// Setting the high bit of each message makes it more secure
	BigInteger highbit=BigInteger(2).pow(bits-1); // ==2<<(bits-1)
	
	// Try some test encryptions
	BigInteger i;
	for (i=0;i<100;i++) {
		BigInteger m=i+highbit; // message
		BigInteger c=m.modPow(e,n); // encrypt
		BigInteger m2=c.modPow(d,n); // decrypt
		std::cout<<m<<" -> "<<c<<" -> "<<m2;
		if (m != m2) std::cout<<"(FAIL!)";
		std::cout<<"\n";
	}
	
	// Print lots of stuff (even stuff you shouldn't!)
	std::cout<<"p="<<p<<" (SHRED!)\n";
	std::cout<<"q="<<q<<" (SHRED!)\n";
	std::cout<<"n="<<n<<"\n";
	std::cout<<"e="<<e<<"\n";
	std::cout<<"d="<<d<<" (SECRET)\n";
	std::cout<<"e*d="<<(e*d).mod((p-1)*(q-1))<<" mod (p-1)*(q-1)\n";
	return 0;
}

